using System.ComponentModel.DataAnnotations;
using System.Threading;
using MediatR;
using Todo.Core.Application.Notifications.Todo;
using Todo.Core.Application.Queries.Todos;
using Todo.Core.Interfaces.Persistence;
using Todo.Core.Interfaces.User;

namespace Todo.Core.Application.Commands.Todo;

public record CreateTodoCommand(
    [Required] string TodoTitle,
    string? TodoProject,
    string? TodoDescription,
    string? Priority) : IRequest<TodoViewModel>
{
    internal class Handler : IRequestHandler<CreateTodoCommand, TodoViewModel>
    {
        private readonly ICurrentUserService _currentUserService;
        private readonly IMediator _mediator;
        private readonly ITodoRepository _todoRepository;

        public Handler(
            ICurrentUserService currentUserService,
            ITodoRepository todoRepository,
            IMediator mediator)
        {
            _currentUserService = currentUserService;
            _todoRepository = todoRepository;
            _mediator = mediator;
        }

        public async Task<TodoViewModel> Handle(
            CreateTodoCommand request,
            CancellationToken cancellationToken)
        {
            var userId = _currentUserService.GetUserId();
            if (userId is null)
                throw new InvalidOperationException("User was not found");

            var todo = await _todoRepository.CreateTodoAsync(
                request.TodoTitle,
                request.TodoProject,
                request.TodoDescription,
                request.Priority,
                userId,
                DateTimeOffset.UtcNow.ToUnixTimeMilliseconds());

            await _mediator.Publish(
                new TodoCreated(todo.Id),
                cancellationToken);

            return TodoViewModel.From(todo);
        }
    }
}